﻿#include "doors.h"
#include <QtXml/qdom.h>
#include <QFile>
#include "../common/sources/system/cgraphics2d.h"
#include "../common/sources/system/filecache.h"
#include "player.h"

CDoors   doors;

//----------------------------------------------------------------------------
CDoors::CDoors()
{

}

//----------------------------------------------------------------------------
// table pour la partie gauche de l'encadrement
SDoorElem   tabLeftFrame[WALL_TABLE_HEIGHT][WALL_TABLE_WIDTH] =
{
    // Rang 3 (le plus loin)
    {
        {"", 0, 0, 0, 0, 0, 0},  // 03
        {"Frame_Far_Left_0.png",  16, 59,  16, 59,  31, 101}, // 13
        {"Frame_Left_0.png",      82, 60,  82, 60,  91, 101}, // 23
        {"Frame_Far_Left_0.png", 141, 60, 147, 60, 150, 102}, // 33
        {"", 0, 0, 0, 0, 0, 0}   // 43
    },

    // Rang 2
    {
        {"", 0, 0, 0, 0, 0, 0},  // 02
        {"", 0, 0, 0, 0, 0, 0},  // 12
        {"Frame_Left_1.png", 65, 54, 65, 54, 82, 118}, // 22
        {"", 0, 0, 0, 0, 0, 0},  // 32
        {"", 0, 0, 0, 0, 0, 0}   // 42
    },

    // Rang 1
    {
        {"", 0, 0, 0, 0, 0, 0},  // 01
        {"", 0, 0, 0, 0, 0, 0},  // 11
        {"Frame_Left_2.png", 44, 46, 44, 46, 68, 139}, // 21
        {"", 0, 0, 0, 0, 0, 0},  // 31
        {"", 0, 0, 0, 0, 0, 0}   // 41
    },

    // Rang 0 (le plus proche)
    {
        {"", 0, 0, 0, 0, 0, 0},  // 00
        {"", 0, 0, 0, 0, 0, 0},  // 10
        {"", 0, 0, 0, 0, 0, 0},  // 20
        {"", 0, 0, 0, 0, 0, 0},  // 30
        {"", 0, 0, 0, 0, 0, 0}   // 40
    }
};

//----------------------------------------------------------------------------
// table pour la partie droite de l'encadrement
SDoorElem   tabRightFrame[WALL_TABLE_HEIGHT][WALL_TABLE_WIDTH] =
{
    // Rang 3 (le plus loin)
    {
        {"", 0, 0, 0, 0, 0, 0},  // 03
        {"Frame_Far_Left_0.png",  67, 60,  73, 60,  76, 102}, // 13
        {"Frame_Left_0.png",     132, 60, 132, 60, 141, 101}, // 23
        {"Frame_Far_Left_0.png", 192, 59, 192, 59, 207, 101}, // 33
        {"", 0, 0, 0, 0, 0, 0}   // 43
    },

    // Rang 2
    {
        {"", 0, 0, 0, 0, 0, 0},  // 02
        {"", 0, 0, 0, 0, 0, 0},  // 12
        {"Frame_Left_1.png", 141, 54, 141, 54, 158, 118}, // 22
        {"", 0, 0, 0, 0, 0, 0},  // 32
        {"", 0, 0, 0, 0, 0, 0}   // 42
    },

    // Rang 1
    {
        {"", 0, 0, 0, 0, 0, 0},  // 01
        {"", 0, 0, 0, 0, 0, 0},  // 11
        {"Frame_Left_2.png", 155, 46, 155, 46, 179, 139}, // 21
        {"", 0, 0, 0, 0, 0, 0},  // 31
        {"", 0, 0, 0, 0, 0, 0}   // 41
    },

    // Rang 0 (le plus proche)
    {
        {"", 0, 0, 0, 0, 0, 0},  // 00
        {"", 0, 0, 0, 0, 0, 0},  // 10
        {"", 0, 0, 0, 0, 0, 0},  // 20
        {"", 0, 0, 0, 0, 0, 0},  // 30
        {"", 0, 0, 0, 0, 0, 0}   // 40
    }
};

//----------------------------------------------------------------------------
// table pour la partie haute de l'encadrement
SDoorElem   tabTopFrame[WALL_TABLE_HEIGHT][WALL_TABLE_WIDTH] =
{
    // Rang 3 (le plus loin)
    {
        {"", 0, 0, 0, 0, 0, 0},  // 03
        {"", 0, 0, 0, 0, 0, 0},  // 13
        {"", 0, 0, 0, 0, 0, 0},  // 23
        {"", 0, 0, 0, 0, 0, 0},  // 33
        {"", 0, 0, 0, 0, 0, 0}   // 43
    },

    // Rang 2
    {
        {"", 0, 0, 0, 0, 0, 0},  // 02
        {"Frame_Top_1.png",   0, 55,   0, 55,  69, 57}, // 12
        {"Frame_Top_1.png",  77, 54,  77, 54, 146, 56}, // 22
        {"Frame_Top_1.png", 154, 55, 154, 55, 223, 57}, // 32
        {"", 0, 0, 0, 0, 0, 0}   // 42
    },

    // Rang 1
    {
        {"", 0, 0, 0, 0, 0, 0},  // 01
        {"Frame_Top_2.png", -53, 47,   0, 47,  31, 50}, // 11
        {"Frame_Top_2.png",  51, 45,  61, 45, 162, 48}, // 21
        {"Frame_Top_2.png", 176, 47, 192, 47, 223, 50}, // 31
        {"", 0, 0, 0, 0, 0, 0}   // 41
    },

    // Rang 0 (le plus proche)
    {
        {"", 0, 0, 0, 0, 0, 0},  // 00
        {"", 0, 0, 0, 0, 0, 0},  // 10
        {"", 0, 0, 0, 0, 0, 0},  // 20
        {"", 0, 0, 0, 0, 0, 0},  // 30
        {"", 0, 0, 0, 0, 0, 0}   // 40
    }
};

//----------------------------------------------------------------------------
// table pour la porte
SDoorElem   tabDoor[WALL_TABLE_HEIGHT][WALL_TABLE_WIDTH] =
{
    // Rang 3 (le plus loin)
    {
        {"", 0, 0, 0, 0, 0, 0},  // 03
        {"0",  29, 62,  30, 62,  72,  99},  // 13
        {"0",  90, 63,  90, 63, 133, 100},  // 23
        {"0", 151, 62, 151, 62, 193,  99},  // 33
        {"", 0, 0, 0, 0, 0, 0}   // 43
    },

    // Rang 2
    {
        {"", 0, 0, 0, 0, 0, 0},  // 02
        {"1",  -3, 56,   0, 56,  58, 114}, // 12
        {"1",  80, 57,  80, 57, 143, 115}, // 22
        {"1", 164, 56, 165, 56, 223, 114}, // 32
        {"", 0, 0, 0, 0, 0, 0}   // 42
    },

    // Rang 1
    {
        {"", 0, 0, 0, 0, 0, 0},  // 01
        {"2", -63, 51,   0, 51,  31, 136}, // 11
        {"2",  64, 49,  64, 49, 159, 134}, // 21
        {"2", 192, 51, 192, 51, 223, 136}, // 31
        {"", 0, 0, 0, 0, 0, 0}   // 41
    },

    // Rang 0 (le plus proche)
    {
        {"", 0, 0, 0, 0, 0, 0},  // 00
        {"", 0, 0, 0, 0, 0, 0},  // 10
        {"", 0, 0, 0, 0, 0, 0},  // 20
        {"", 0, 0, 0, 0, 0, 0},  // 30
        {"", 0, 0, 0, 0, 0, 0}   // 40
    }
};

//----------------------------------------------------------------------------
void    CDoors::drawFrameElement(QImage* image, CVec2 tablePos, SDoorElem table[WALL_TABLE_HEIGHT][WALL_TABLE_WIDTH], bool isFlipped)
{
    SDoorElem&  cell = table[tablePos.y][tablePos.x];

    if (cell.file[0] != 0)
    {
        std::string fileName = std::string("gfx/3DView/doors/") + std::string(cell.file);
        QImage  elemImage = fileCache.getImage(fileName);
        CVec2 pos(cell.x, cell.y);
        QRect clip(cell.cx1, cell.cy1,
                   cell.cx2 - cell.cx1 + 1, cell.cy2 - cell.cy1 + 1);
        graph2D.drawImage(image, pos, elemImage, 0, isFlipped, clip);
    }
}

//----------------------------------------------------------------------------
void    CDoors::drawDoor(QImage* image, CVec2 tablePos, std::string fileName)
{
    SDoorElem&  cell = tabDoor[tablePos.y][tablePos.x];

    if (cell.file[0] != 0)
    {
        std::string doorName = std::string("gfx/3DView/doors/") + fileName;
        QImage  elemImage = fileCache.getImage(doorName);
        CVec2 pos(cell.x, cell.y);
        QRect clip(cell.cx1, cell.cy1,
                   cell.cx2 - cell.cx1 + 1, cell.cy2 - cell.cy1 + 1);
        graph2D.drawImage(image, pos, elemImage, 0, false, clip);
    }
}

//----------------------------------------------------------------------------
bool CDoors::isFacing(CTile* tile)
{
    EWallSide playerSide = player.getWallSide(eWallSideUp);

    return (tile->mWalls[playerSide].getType() != eWallDoorSide);
}

//----------------------------------------------------------------------------
void CDoors::draw(QImage* image, CTile* tile, CVec2 tablePos)
{
    if (isFacing(tile) == true)
    {
        drawFrameElement(image, tablePos, tabTopFrame, false);
        drawFrameElement(image, tablePos, tabLeftFrame, false);
        drawFrameElement(image, tablePos, tabRightFrame, true);

        SDoorElem&  cell = tabDoor[tablePos.y][tablePos.x];
        if (cell.file[0] != 0)
        {
            int doorType = tile->getDoorParam("Type");
            int doorNum = cell.file[0] - '0';
            std::string fileName = doorsDatas[doorType].files[doorNum].toUtf8().constData();
            drawDoor(image, tablePos, fileName);
        }
    }
}

//---------------------------------------------------------------------------------------------
void    CDoors::readDoorsDB()
{
    QDomDocument doc;

    QFile f("databases/doors.xml");
    f.open(QIODevice::ReadOnly);
    doc.setContent(&f);
    f.close();

    QDomElement root = doc.documentElement();
    QDomElement door = root.firstChild().toElement();

    while(!door.isNull())
    {
        std::vector<CParamType> paramsTypesList;
        CDoorData   newDoor;

        if (door.tagName() == "door")
        {
            QDomElement doorInfo = door.firstChild().toElement();

            while(!doorInfo.isNull())
            {
                if (doorInfo.tagName() == "image1")
                {
                    newDoor.files[0] = doorInfo.text();
                }
                else if (doorInfo.tagName() == "image2")
                {
                    newDoor.files[1] = doorInfo.text();
                }
                else if (doorInfo.tagName() == "image3")
                {
                    newDoor.files[2] = doorInfo.text();
                }

                doorInfo = doorInfo.nextSibling().toElement();
            }
            doorsDatas.push_back(newDoor);
        }
        door = door.nextSibling().toElement();
    }
}
